package pwc.taxtech.atms.security;

import io.jsonwebtoken.impl.DefaultClaims;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.authentication.AuthenticationCredentialsNotFoundException;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.web.authentication.AbstractAuthenticationProcessingFilter;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

public class JwtAuthenticationFilter extends AbstractAuthenticationProcessingFilter {

    @Value("${jwt.expireSecond}")
    private Integer jwtExpireSecond;
    @Value("${jwt.refreshSecond}")
    private Integer jwtRefreshSecond;
    @Autowired
    private JwtUtil jwtUtil;

    public JwtAuthenticationFilter() {
        super("/**");
    }

    protected boolean requiresAuthentication(HttpServletRequest request, HttpServletResponse response) {
        String header = request.getHeader("Authorization");
        return (header != null && (header.startsWith("bearer ") || header.startsWith("Bearer ")));
    }

    @Override
    public Authentication attemptAuthentication(HttpServletRequest request, HttpServletResponse response) {

        String header = request.getHeader("Authorization");

        if (header == null || !(header.startsWith("bearer ") || header.startsWith("Bearer "))) {
            throw new AuthenticationCredentialsNotFoundException("No JWT token found in request headers");
        }

        String authToken = header.substring(7);
        if (!StringUtils.hasText(authToken)) {
            throw new AuthenticationCredentialsNotFoundException("Empty Bearer token");
        }

        JwtAuthenticationToken authRequest = new JwtAuthenticationToken(authToken);
        
        setDetails(request, authRequest);

        return getAuthenticationManager().authenticate(authRequest);
    }
    
    protected void setDetails(HttpServletRequest request,
            UsernamePasswordAuthenticationToken authRequest) {
        authRequest.setDetails(authenticationDetailsSource.buildDetails(request));
    }

    @Override
    protected void successfulAuthentication(HttpServletRequest request, HttpServletResponse response, FilterChain chain,
            Authentication authResult) throws IOException, ServletException {
        super.successfulAuthentication(request, response, chain, authResult);

        // As this authentication is in HTTP header, after success we need to continue
        // the request normally
        // and return the response as if the resource was not secured at all
        try {
            JwtUser jwtUser = (JwtUser) authResult.getPrincipal();
            if (null != jwtUser) {
                DefaultClaims claims = jwtUser.getDefaultClaims();
                if (claims.getExpiration().getTime() - System.currentTimeMillis() <= jwtRefreshSecond * 1000) {
                    String newToken = jwtUtil.generateToken(jwtUser.getUsername(), jwtUser.getDatabaseUsername(),
                            jwtUser.getUserid());
                    response.setHeader("Access-Control-Expose-Headers", "refreshToken");
                    response.setHeader("refreshToken", newToken);
                    logger.debug("refreshToken: " + newToken);
                }
            }
        } catch (Exception e) {
            logger.error("", e);
        }
        chain.doFilter(request, response);
    }

    @Override
    protected void unsuccessfulAuthentication(HttpServletRequest request, HttpServletResponse response, AuthenticationException failed) throws IOException, ServletException {
        super.unsuccessfulAuthentication(request, response, failed);
        response.setStatus(HttpServletResponse.SC_UNAUTHORIZED);
    }
}