package pwc.taxtech.atms.service.impl;

import com.google.common.collect.Lists;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.openxml4j.exceptions.InvalidFormatException;
import org.apache.poi.ss.usermodel.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;
import pwc.taxtech.atms.common.message.ErrorMessageCN;
import pwc.taxtech.atms.common.util.DateUtils;
import pwc.taxtech.atms.constant.enums.EnumCitImportType;
import pwc.taxtech.atms.constant.enums.EnumImportType;
import pwc.taxtech.atms.constant.enums.EnumTbImportType;
import pwc.taxtech.atms.dao.*;
import pwc.taxtech.atms.dto.CitDataImportLogDto;
import pwc.taxtech.atms.dto.OperationResultDto;
import pwc.taxtech.atms.entity.*;
import pwc.taxtech.atms.exception.ServiceException;

import javax.annotation.Resource;
import java.io.IOException;
import java.io.InputStream;
import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

@Service
public class CitImportExcelServiceImpl extends BaseService {

    @Resource
    private OrganizationMapper organizationMapper;

    @Autowired
    private CitDataImportLogMapper citDataImportLogMapper;

    @Autowired
    private CitJournalEntryAdjustMapper citJournalEntryAdjustMapper;

    @Autowired
    private CitTrialBalanceMapper citTrialBalanceMapper;

    @Autowired
    private CitDraftAccountMappingMapper citDAMappingMapper;

    @Autowired
    private CitBalanceSheetPrcAdjustMapper citBSPrcAdjustMapper;

    @Autowired
    private CitProfitPrcAdjustMapper citProfitPrcAdjustMapper;

    @Autowired
    private CitEAMAssetsDisposalMapper citEAMAssetsMapper;

    @Autowired
    private CitSalaryAdvanceMapper citSalaryAdvanceMapper;

    @Autowired
    private FileService fileService;

    /**
     * 根据导入的文件类型进行方法分发
     *
     * @param file
     * @param fileName
     * @param periodDate
     * @param importType
     * @param importFileType
     * @return
     * @throws IOException
     * @throws InvalidFormatException
     * @throws ParseException
     */
    public OperationResultDto citImportExcel(MultipartFile file,List<String> orgList, String fileName, String periodDate, Integer importType, Integer importFileType) throws IOException, InvalidFormatException, ParseException {
        OperationResultDto operationResultDto = new OperationResultDto();
        InputStream input = file.getInputStream();

        if (StringUtils.isBlank(periodDate) || "null".equals(periodDate)) {
            throw new ServiceException(ErrorMessageCN.DoNotSelectPeriod);
        }
        Integer period = DateUtils.strToPeriod(periodDate);

        //获取上传文件类型的枚举值
        EnumCitImportType enumValue = EnumCitImportType.valueOf(importFileType);
        switch (enumValue) {
            case JournalAdjust:
                operationResultDto = importJournalAdjust(input, orgList, fileName, period, importType);
                break;
            case TrialBalance:
                operationResultDto = importTrialBalance(input, orgList, fileName, period, importType);
                break;
            case DraftAccountMapping:
                operationResultDto = importDraftAccountMapping(input, orgList, fileName, period, importType);
                break;
            case BalanceSheetPrcAdjust:
                operationResultDto = importBalanceSheetPrcAdjust(input, orgList, fileName, period, importType);
                break;
            case ProfitPrcAdjust:
                operationResultDto = importProfitPrcAdjust(input, orgList, fileName, period, importType);
                break;
            case EAMAssetsDisposal:
                operationResultDto = importEAMAssetsDisposal(input, orgList, fileName, period, importType);
                break;
            case SalaryAdvance:
                operationResultDto = importSalaryAdvance(input, orgList, fileName, period, importType);
                break;
        }
        return operationResultDto;
    }

    /**
     * 导入日记账调整版
     *
     * @param inputStream
     * @param fileName
     * @param importType
     * @return
     * @throws IOException
     * @throws InvalidFormatException
     * @throws ParseException
     */
    public OperationResultDto importJournalAdjust(InputStream inputStream, List<String> orgList, String fileName, Integer period, Integer importType) throws IOException, InvalidFormatException, ParseException {
        OperationResultDto<Object> saveResult = new OperationResultDto<>();
        Workbook workbook = fileService.getWorkbook(inputStream, fileName, "citJournalAdjust");
        if (workbook == null) {
            saveResult.setResult(false);
            saveResult.setResultMsg("读取Excel出现内部错误");
            return saveResult;
        }
        //目前日记账调整版导入只有一个Sheet页，所以只取第一个Sheet即可
        Sheet sheet = workbook.getSheetAt(0);
        if (sheet == null) {
            saveResult.setResult(false);
            saveResult.setResultMsg("读取Excel出现内部错误");
            return saveResult;
        }

        List<CitJournalEntryAdjust> journalAdjustList = new ArrayList<>();
        List<CitDataImportLog> citDataImportLogList = new ArrayList<>();
//        Integer period = CitCommonUtil.getPeriod();
        String companyName = CitCommonUtil.getValue(sheet.getRow(1).getCell(19)).toString();
        String companyCode = CitCommonUtil.getValue(sheet.getRow(1).getCell(9)).toString();
        ;
        List<Organization> organizations = getOrganizationByCode(companyCode);
        String orgId = "";
        String taxPayerId = "";
        if (organizations != null && organizations.size() > 0) {
            orgId = organizations.get(0).getId();
            taxPayerId = StringUtils.isNotEmpty(organizations.get(0).getLegalCode()) ?
                    organizations.get(0).getLegalCode() : organizations.get(0).getTaxPayerNumber();

            if(!orgList.contains(orgId)){
                saveResult.setResult(false);
                saveResult.setResultMsg(ErrorMessageCN.DoNotSelectCompany);
                return saveResult;
            }
        }else{
            saveResult.setResult(false);
            saveResult.setResultMsg(ErrorMessageCN.NoCompanyError);
            return saveResult;
        }


        //通过循环来完成sheet获取每一行每一列数据并进行相关逻辑处理
        for (int rowNum = sheet.getFirstRowNum() + 1; rowNum <= sheet.getLastRowNum(); rowNum++) {

            //获取该行数据
            Row rowData = sheet.getRow(rowNum);


            CitJournalEntryAdjust citJournal = new CitJournalEntryAdjust();
            citJournal.setId(idService.nextId());
            citJournal.setPeriod(period);
            citJournal.setOrganizationId(orgId);
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            citJournal.setAccountPeriod(new Integer(CitCommonUtil.getValue(rowData.getCell(2)).toString().replace("-", "")));
            citJournal.setAccountingDate(sdf.parse(CitCommonUtil.getValue(rowData.getCell(3)).toString()));
            citJournal.setVoucherNum(CitCommonUtil.getValue(rowData.getCell(7)).toString());
            citJournal.setDescription(CitCommonUtil.getValue(rowData.getCell(8)).toString());
            citJournal.setSegment1(CitCommonUtil.getValue(rowData.getCell(9)).toString());
            citJournal.setSegment3(CitCommonUtil.getValue(rowData.getCell(11)).toString());
            citJournal.setSegment1Name(CitCommonUtil.getValue(rowData.getCell(19)).toString());
            citJournal.setSegment3Name(CitCommonUtil.getValue(rowData.getCell(21)).toString());
            Object cellvalue = CitCommonUtil.getValue(rowData.getCell(31));
            if(cellvalue == null){
                cellvalue = "0";
            }
            citJournal.setAccountedDr(new BigDecimal(cellvalue.toString()));
            cellvalue = CitCommonUtil.getValue(rowData.getCell(32));
            if(cellvalue == null){
                cellvalue = "0";
            }
            citJournal.setAccountedCr(new BigDecimal(cellvalue.toString()));

            journalAdjustList.add(citJournal);
        }

        String msg = "追加导入成功";
        if (EnumTbImportType.CoverImport.getCode().equals(importType)) {
            msg = "覆盖导入成功";
            CitJournalEntryAdjustExample example = new CitJournalEntryAdjustExample();
            CitJournalEntryAdjustExample.Criteria criteria = example.createCriteria();
            criteria.andCreatedByEqualTo(authUserHelper.getCurrentAuditor().get());
            criteria.andPeriodEqualTo(period);
            citJournalEntryAdjustMapper.deleteByExample(example);
            updateImportLog(1);
        }
        int insertBatchNum = citJournalEntryAdjustMapper.insertBatch(journalAdjustList);
        CitDataImportLog citDataImportLog = generalCitDataImportLog(companyCode, orgId, taxPayerId,
                EnumCitImportType.JournalAdjust.getCode(), period / 100,
                0, period % 100,
                companyName, "日记账调整",true);
        citDataImportLog.setRecordSize(insertBatchNum);
        citDataImportLog.setErrorMsg(msg);

        citDataImportLogList.add(citDataImportLog);
        addDataImportLog(citDataImportLogList);
        saveResult.setResult(true);
        saveResult.setResultMsg("导入成功");
        return saveResult;
    }

    /**
     * 导入试算平衡表
     *
     * @param inputStream
     * @param fileName
     * @param importType
     * @return
     * @throws IOException
     * @throws InvalidFormatException
     * @throws ParseException
     */
    public OperationResultDto importTrialBalance(InputStream inputStream, List<String> orgList, String fileName, Integer period, Integer importType) throws IOException, InvalidFormatException, ParseException {
        OperationResultDto<Object> saveResult = new OperationResultDto<>();
        Workbook workbook = fileService.getWorkbook(inputStream, fileName, "citTrialBalance");
        if (workbook == null) {
            saveResult.setResult(false);
            saveResult.setResultMsg("读取Excel出现内部错误");
            return saveResult;
        }
        //目前资产导入只有一个Sheet页，所以只取第一个Sheet即可
        Sheet sheet = workbook.getSheetAt(0);
        if (sheet == null) {
            saveResult.setResult(false);
            saveResult.setResultMsg("读取Excel出现内部错误");
            return saveResult;
        }

        List<CitTrialBalance> trialBalanceList = new ArrayList<>();
        List<CitDataImportLog> citDataImportLogList = new ArrayList<>();
        //获取当前期间
//        Integer period = CitCommonUtil.getPeriod();

        Map<String, Integer> companySuccessMap = new HashMap();
        Map<String, Integer> companyFailMap = new HashMap();
        Map<String, Integer> noCompanyMap = new HashMap();
        //通过循环来完成sheet获取每一行每一列数据并进行相关逻辑处理
        for (int rowNum = sheet.getFirstRowNum() + 8; rowNum <= sheet.getLastRowNum(); rowNum++) {

            //获取该行数据
            Row rowData = sheet.getRow(rowNum);

            CitTrialBalance trialBalance = new CitTrialBalance();
            trialBalance.setId(idService.nextId());
            trialBalance.setPeriod(period);

            CitDataImportLog citDataImportLogError = generalCitDataImportLog("", "", "",
                    EnumCitImportType.TrialBalance.getCode(), 0,
                    0, 0,
                    "", "试算平衡表", false);
            citDataImportLogError.setRecordSize(1);

            Object cellValue = CitCommonUtil.getValue(rowData.getCell(5));
            if(cellValue == null){
                citDataImportLogError.setErrorMsg("数据错误，第"+(rowNum+1)+"行主体为空");
                citDataImportLogList.add(citDataImportLogError);
                continue;
            }

            String companyCode = String.valueOf((int)(double)CitCommonUtil.getValue(rowData.getCell(5)));
            //根据code（主体）获取机构相关信息
            List<Organization> organizations = getOrganizationByCode(companyCode);
            String companyName = "";
            String orgId = "";
            String taxPayerId = "";
            if (organizations != null && organizations.size() > 0) {

                orgId = organizations.get(0).getId();
                taxPayerId = StringUtils.isNotEmpty(organizations.get(0).getLegalCode()) ?
                        organizations.get(0).getLegalCode() : organizations.get(0).getTaxPayerNumber();
                companyName = organizations.get(0).getName();

                //拼接公司相关信息，作为键，其Excel出现的数量作为值
                StringBuilder sb = new StringBuilder(companyCode);
                sb.append(",");
                sb.append(orgId);
                sb.append(",");
                sb.append(taxPayerId);
                sb.append(",");
                sb.append(companyName);
                if(!orgList.contains(orgId)){
                    companyFailMap.put(sb.toString(),companyFailMap.get(sb.toString())==null?1:companyFailMap.get(sb.toString())+1);
                    continue;
                }
                companySuccessMap.put(sb.toString(),companySuccessMap.get(sb.toString())==null?1:companySuccessMap.get(sb.toString())+1);

            }else{
                noCompanyMap.put(companyCode,companySuccessMap.get(companyCode)==null?1:companySuccessMap.get(companyCode)+1);
                continue;
            }

            trialBalance.setOrganizationId(orgId);
            trialBalance.setAccountCode(CitCommonUtil.getValue(rowData.getCell(7)).toString());
            trialBalance.setAccountDescription(CitCommonUtil.getValue(rowData.getCell(17)).toString());
            //此期间默认为12
            trialBalance.setAccountPeriod(12);
            trialBalance.setBeginningBalance(new BigDecimal(CitCommonUtil.getValue(rowData.getCell(25)).toString()));
            trialBalance.setEndingBalance(new BigDecimal(CitCommonUtil.getValue(rowData.getCell(39)).toString()));

            trialBalanceList.add(trialBalance);
        }
        //判断导入的方式
        String msg = "追加导入成功";
        if (EnumTbImportType.CoverImport.getCode().equals(importType)) {
            //覆盖导入要删除原有的该操作人在当前期间导入的数据，且更改之前导入的日志记录状态
            msg = "覆盖导入成功";
            CitTrialBalanceExample citTrialBalanceExample = new CitTrialBalanceExample();
            CitTrialBalanceExample.Criteria criteria = citTrialBalanceExample.createCriteria();
            criteria.andCreateByEqualTo(authUserHelper.getCurrentAuditor().get());
            criteria.andPeriodEqualTo(period);
            citTrialBalanceMapper.deleteByExample(citTrialBalanceExample);
            updateImportLog(1);
        }
        int insertBatchNum = citTrialBalanceMapper.insertBatch(trialBalanceList);

        //循环遍历成功导入的数据，组装日志记录实体并放入集合
        for (Map.Entry<String, Integer> entry : companySuccessMap.entrySet()) {
            String[] split = entry.getKey().split(",");
            CitDataImportLog citDataImportLog = generalCitDataImportLog(split[0], split[1], split[2],
                    EnumCitImportType.TrialBalance.getCode(), period / 100,
                    0, period % 100,
                    split[3], "试算平衡表", true);
            citDataImportLog.setRecordSize(entry.getValue());
            citDataImportLog.setErrorMsg(msg);
            citDataImportLogList.add(citDataImportLog);
        }
        for (Map.Entry<String, Integer> entry : companyFailMap.entrySet()) {
            String[] split = entry.getKey().split(",");
            CitDataImportLog citDataImportLogError = generalCitDataImportLog(split[0], split[1], split[2],
                    EnumCitImportType.TrialBalance.getCode(), period / 100,
                    0, period % 100,
                    split[3], "试算平衡表", false);
            citDataImportLogError.setRecordSize(entry.getValue());
            citDataImportLogError.setErrorMsg(ErrorMessageCN.DoNotSelectCompany);
            citDataImportLogList.add(citDataImportLogError);
        }
        for (Map.Entry<String, Integer> entry : noCompanyMap.entrySet()) {
            CitDataImportLog citDataImportLogError = generalCitDataImportLog(entry.getKey(), "", "",
                    EnumCitImportType.TrialBalance.getCode(), 0,
                    0, 0,
                    "", "试算平衡表", false);
            citDataImportLogError.setRecordSize(entry.getValue());
            citDataImportLogError.setErrorMsg(ErrorMessageCN.NoCompanyError);
            citDataImportLogList.add(citDataImportLogError);
        }

        addDataImportLog(citDataImportLogList);
        saveResult.setResult(true);
        saveResult.setResultMsg("导入成功");
        return saveResult;
    }

    /**
     * 导入底稿&科目Mapping表
     * @param inputStream
     * @param fileName
     * @param period
     * @param importType
     * @return
     * @throws IOException
     * @throws InvalidFormatException
     * @throws ParseException
     */
    public OperationResultDto importDraftAccountMapping(InputStream inputStream, List<String> orgList, String fileName, Integer period, Integer importType) throws IOException, InvalidFormatException, ParseException {
        OperationResultDto<Object> saveResult = new OperationResultDto<>();
        Workbook workbook = fileService.getWorkbook(inputStream, fileName, "citDraftAccountMapping");
        if (workbook == null) {
            saveResult.setResult(false);
            saveResult.setResultMsg("读取Excel出现内部错误");
            return saveResult;
        }
        //目前资产导入只有一个Sheet页，所以只取第一个Sheet即可
        Sheet sheet = workbook.getSheetAt(0);
        if (sheet == null) {
            saveResult.setResult(false);
            saveResult.setResultMsg("读取Excel出现内部错误");
            return saveResult;
        }

        List<CitDraftAccountMapping> citDAMappings = new ArrayList<>();
        List<CitDataImportLog> citDataImportLogList = new ArrayList<>();
//        Integer period = CitCommonUtil.getPeriod();

        //通过循环来完成sheet获取每一行每一列数据并进行相关逻辑处理
        for (int rowNum = sheet.getFirstRowNum() + 1; rowNum <= sheet.getLastRowNum(); rowNum++) {

            CitDataImportLog citDataImportLogError = generalCitDataImportLog("", "", "",
                    EnumCitImportType.DraftAccountMapping.getCode(), 0,
                    0, 0,
                    "", "底稿&科目Mapping表", false);
            citDataImportLogError.setRecordSize(1);
            //获取该行数据
            Row rowData = sheet.getRow(rowNum);

            CitDraftAccountMapping citDAMapping = new CitDraftAccountMapping();
            citDAMapping.setId(idService.nextId());
            citDAMapping.setPeriod(period);
            Object cellValue = CitCommonUtil.getValue(rowData.getCell(5));
            if(cellValue == null){
                citDataImportLogError.setErrorMsg("数据错误，第"+(rowNum+1)+"行科目代码为空");
                citDataImportLogList.add(citDataImportLogError);
                continue;
            }
            citDAMapping.setAcctCode(CitCommonUtil.getValue(rowData.getCell(5)).toString());

            cellValue = CitCommonUtil.getValue(rowData.getCell(6));
            if(cellValue == null){
                citDataImportLogError.setErrorMsg("第"+(rowNum+1)+"行科目名称为空");
                citDataImportLogList.add(citDataImportLogError);
                continue;
            }
            citDAMapping.setAccountName(CitCommonUtil.getValue(rowData.getCell(6)).toString());

            cellValue = CitCommonUtil.getValue(rowData.getCell(3));
            if(cellValue == null){
                citDataImportLogError.setErrorMsg("第"+(rowNum+1)+"行小类为空");
                citDataImportLogList.add(citDataImportLogError);
                continue;
            }
            citDAMapping.setAttribute(cellValue.toString());
            citDAMappings.add(citDAMapping);
        }

        String msg = "追加导入成功";
        if (EnumTbImportType.CoverImport.getCode().equals(importType)) {
            msg = "覆盖导入成功";
            CitDraftAccountMappingExample example = new CitDraftAccountMappingExample();
            CitDraftAccountMappingExample.Criteria criteria = example.createCriteria();
            criteria.andCreatedByEqualTo(authUserHelper.getCurrentAuditor().get());
            criteria.andPeriodEqualTo(period);
            citDAMappingMapper.deleteByExample(example);
            updateImportLog(1);
        }

        int insertBatchNum = citDAMappingMapper.insertBatch(citDAMappings);
        CitDataImportLog citDataImportLog = generalCitDataImportLog("", "", "",
                EnumCitImportType.DraftAccountMapping.getCode(), period / 100,
                0, period % 100,
                "", "底稿&科目Mapping表", true);
        citDataImportLog.setRecordSize(insertBatchNum);
        citDataImportLog.setErrorMsg(msg);
        citDataImportLogList.add(citDataImportLog);
        addDataImportLog(citDataImportLogList);
        saveResult.setResult(true);
        saveResult.setResultMsg("导入成功");
        return saveResult;
    }

    /**
     * 导入资产负债表（单家PRC）
     * @param inputStream
     * @param fileName
     * @param period
     * @param importType
     * @return
     * @throws IOException
     * @throws InvalidFormatException
     * @throws ParseException
     */
    public OperationResultDto importBalanceSheetPrcAdjust(InputStream inputStream, List<String> orgList, String fileName, Integer period, Integer importType) throws IOException, InvalidFormatException, ParseException {
        OperationResultDto<Object> saveResult = new OperationResultDto<>();
        Workbook workbook = fileService.getWorkbook(inputStream, fileName, "citTrialBalance");
        if (workbook == null) {
            saveResult.setResult(false);
            saveResult.setResultMsg("读取Excel出现内部错误");
            return saveResult;
        }
        //目前资产导入只有一个Sheet页，所以只取第一个Sheet即可
        Sheet sheet = workbook.getSheetAt(0);
        if (sheet == null) {
            saveResult.setResult(false);
            saveResult.setResultMsg("读取Excel出现内部错误");
            return saveResult;
        }

        List<CitBalanceSheetPrcAdjust> citBSPrcAdjustList = new ArrayList<>();
        List<CitDataImportLog> citDataImportLogList = new ArrayList<>();
//        Integer period = CitCommonUtil.getPeriod();
        String companyName = CitCommonUtil.getValue(sheet.getRow(5).getCell(1)).toString();
        List<Organization> organizations = getOrganizationByName(companyName);
        String companyCode = "";
        String orgId = "";
        String taxPayerId = "";
        if (organizations != null && organizations.size() > 0) {
            companyCode = organizations.get(0).getCode();
            orgId = organizations.get(0).getId();

            if(!orgList.contains(orgId)){
                saveResult.setResult(false);
                saveResult.setResultMsg(ErrorMessageCN.DoNotSelectCompany);
                return saveResult;
            }

            taxPayerId = StringUtils.isNotEmpty(organizations.get(0).getLegalCode()) ?
                    organizations.get(0).getLegalCode() : organizations.get(0).getTaxPayerNumber();

        }else{
            saveResult.setResult(false);
            saveResult.setResultMsg(ErrorMessageCN.NoCompanyError);
            return saveResult;
        }

        //通过循环来完成sheet获取每一行每一列数据并进行相关逻辑处理
        for (int rowNum = sheet.getFirstRowNum() + 10; rowNum <= sheet.getLastRowNum(); rowNum++) {

            //获取该行数据
            Row rowData = sheet.getRow(rowNum);

            CitDataImportLog citDataImportLogError = generalCitDataImportLog("", "", "",
                    EnumCitImportType.BalanceSheetPrcAdjust.getCode(), 0,
                    0, 0,
                    "", "资产负债表(单家PRC)",  false);
            citDataImportLogError.setRecordSize(1);

            CitBalanceSheetPrcAdjust citBSPrcAdjust = new CitBalanceSheetPrcAdjust();
            citBSPrcAdjust.setId(idService.nextId());
            citBSPrcAdjust.setPeriod(period);
            citBSPrcAdjust.setOrganizationId(orgId);

            Object cellValue = CitCommonUtil.getValue(rowData.getCell(0));
            if(cellValue == null){
                citDataImportLogError.setErrorMsg("第"+(rowNum+1)+"行项目名称为空");
                citDataImportLogList.add(citDataImportLogError);
                continue;
            }
            citBSPrcAdjust.setItemName(cellValue.toString());

            cellValue = CitCommonUtil.getValue(rowData.getCell(1));
            if(cellValue == null){
                citDataImportLogError.setErrorMsg("第"+(rowNum+1)+"行期初余额为空");
                citDataImportLogList.add(citDataImportLogError);
                continue;
            }
            System.out.println(cellValue);
            citBSPrcAdjust.setBegBal(new BigDecimal(cellValue.toString().replace(",","")));


            cellValue = CitCommonUtil.getValue(rowData.getCell(2));
            if(cellValue == null){
                citDataImportLogError.setErrorMsg("第"+(rowNum+1)+"行期末余额为空");
                citDataImportLogList.add(citDataImportLogError);
                continue;
            }
            citBSPrcAdjust.setEndBal(new BigDecimal(cellValue.toString().replace(",","")));
            citBSPrcAdjustList.add(citBSPrcAdjust);
        }

        String msg = "追加导入成功";
        if (EnumTbImportType.CoverImport.getCode().equals(importType)) {
            msg = "覆盖导入成功";
            CitBalanceSheetPrcAdjustExample example = new CitBalanceSheetPrcAdjustExample();
            CitBalanceSheetPrcAdjustExample.Criteria criteria = example.createCriteria();
            criteria.andCreateByEqualTo(authUserHelper.getCurrentAuditor().get());
            criteria.andPeriodEqualTo(period);
            citBSPrcAdjustMapper.deleteByExample(example);
            updateImportLog(1);
        }

        int insertBatchNum = citBSPrcAdjustMapper.insertBatch(citBSPrcAdjustList);
        CitDataImportLog citDataImportLog = generalCitDataImportLog(companyCode, orgId, taxPayerId,
                EnumCitImportType.BalanceSheetPrcAdjust.getCode(), period / 100,
                0, period % 100,
                companyName, "资产负债表(单家PRC)", true);
        citDataImportLog.setRecordSize(insertBatchNum);
        citDataImportLog.setErrorMsg(msg);
        citDataImportLogList.add(citDataImportLog);
        addDataImportLog(citDataImportLogList);
        return null;
    }

    /**
     * 导入利润表（单家PRC）
     * @param inputStream
     * @param fileName
     * @param period
     * @param importType
     * @return
     * @throws IOException
     * @throws InvalidFormatException
     * @throws ParseException
     */
    public OperationResultDto importProfitPrcAdjust(InputStream inputStream, List<String> orgList, String fileName, Integer period, Integer importType) throws IOException, InvalidFormatException, ParseException {
        OperationResultDto<Object> saveResult = new OperationResultDto<>();
        Workbook workbook = fileService.getWorkbook(inputStream, fileName, "citTrialBalance");
        if (workbook == null) {
            saveResult.setResult(false);
            saveResult.setResultMsg("读取Excel出现内部错误");
            return saveResult;
        }
        //目前资产导入只有一个Sheet页，所以只取第一个Sheet即可
        Sheet sheet = workbook.getSheetAt(0);
        if (sheet == null) {
            saveResult.setResult(false);
            saveResult.setResultMsg("读取Excel出现内部错误");
            return saveResult;
        }

        List<CitProfitPrcAdjust> citProfitPrcAdjustList = new ArrayList<>();
        List<CitDataImportLog> citDataImportLogList = new ArrayList<>();
//        Integer period = CitCommonUtil.getPeriod();
        String companyName = CitCommonUtil.getValue(sheet.getRow(4).getCell(1)).toString();
        String companyCode = CitCommonUtil.getValue(sheet.getRow(5).getCell(1)).toString();
        List<Organization> organizations = getOrganizationByCode(companyCode);
        String orgId = "";
        String taxPayerId = "";
        if (organizations != null && organizations.size() > 0) {
            orgId = organizations.get(0).getId();

            if(!orgList.contains(orgId)){
                saveResult.setResult(false);
                saveResult.setResultMsg(ErrorMessageCN.DoNotSelectCompany);
                return saveResult;
            }

            taxPayerId = StringUtils.isNotEmpty(organizations.get(0).getLegalCode()) ?
                    organizations.get(0).getLegalCode() : organizations.get(0).getTaxPayerNumber();

        }else{
            saveResult.setResult(false);
            saveResult.setResultMsg(ErrorMessageCN.DoNotSelectCompany);
            return saveResult;
        }

        //通过循环来完成sheet获取每一行每一列数据并进行相关逻辑处理
        for (int rowNum = sheet.getFirstRowNum() + 10; rowNum <= sheet.getLastRowNum(); rowNum++) {

            //获取该行数据
            Row rowData = sheet.getRow(rowNum);

            CitDataImportLog citDataImportLogError = generalCitDataImportLog(companyCode, orgId, taxPayerId,
                    EnumCitImportType.ProfitPrcAdjust.getCode(), 0,
                    0, 0,
                    companyName, "利润表(单家PRC)",  false);
            citDataImportLogError.setRecordSize(1);

            CitProfitPrcAdjust citProfitPrcAdjust = new CitProfitPrcAdjust();
            citProfitPrcAdjust.setId(idService.nextId());
            citProfitPrcAdjust.setPeriod(period);

            Object cellValue = CitCommonUtil.getValue(rowData.getCell(0));
            if(cellValue == null){
                citDataImportLogError.setErrorMsg("第"+(rowNum+1)+"行项目名称为空");
                citDataImportLogList.add(citDataImportLogError);
                continue;
            }
            citProfitPrcAdjust.setItemName(CitCommonUtil.getValue(rowData.getCell(0)).toString());

            cellValue = CitCommonUtil.getValue(rowData.getCell(1));
            if(cellValue == null){
                citDataImportLogError.setErrorMsg("第"+(rowNum+1)+"行当期发生为空");
                citDataImportLogList.add(citDataImportLogError);
                continue;
            }
            citProfitPrcAdjust.setPeriodAmt(new BigDecimal(cellValue.toString().replace(",","")));

            cellValue = CitCommonUtil.getValue(rowData.getCell(2));
            if(cellValue == null){
                citDataImportLogError.setErrorMsg("第"+(rowNum+1)+"行本年累计为空");
                citDataImportLogList.add(citDataImportLogError);
                continue;
            }
            citProfitPrcAdjust.setYtdAmt(new BigDecimal(cellValue.toString().replace(",","")));
            citProfitPrcAdjustList.add(citProfitPrcAdjust);
        }

        String msg = "追加导入成功";
        if (EnumTbImportType.CoverImport.getCode().equals(importType)) {
            msg = "覆盖导入成功";
            CitProfitPrcAdjustExample example = new CitProfitPrcAdjustExample();
            CitProfitPrcAdjustExample.Criteria criteria = example.createCriteria();
            criteria.andCreateByEqualTo(authUserHelper.getCurrentAuditor().get());
            criteria.andPeriodEqualTo(period);
            citProfitPrcAdjustMapper.deleteByExample(example);
            updateImportLog(1);
        }

        int insertBatchNum = citProfitPrcAdjustMapper.insertBatch(citProfitPrcAdjustList);
        CitDataImportLog citDataImportLog = generalCitDataImportLog(companyCode, orgId, taxPayerId,
                EnumCitImportType.ProfitPrcAdjust.getCode(), period / 100,
                0, period % 100,
                companyName, "利润表(单家PRC)", true);
        citDataImportLog.setRecordSize(insertBatchNum);
        citDataImportLog.setErrorMsg(msg);
        citDataImportLogList.add(citDataImportLog);
        addDataImportLog(citDataImportLogList);
        return null;
    }

    /**
     * 导入EAM资产处置金额记录表
     * @param inputStream
     * @param fileName
     * @param period
     * @param importType
     * @return
     * @throws IOException
     * @throws InvalidFormatException
     * @throws ParseException
     */
    public OperationResultDto importEAMAssetsDisposal(InputStream inputStream, List<String> orgList, String fileName, Integer period, Integer importType) throws IOException, InvalidFormatException, ParseException {
        OperationResultDto<Object> saveResult = new OperationResultDto<>();
        Workbook workbook = fileService.getWorkbook(inputStream, fileName, "citTrialBalance");
        if (workbook == null) {
            saveResult.setResult(false);
            saveResult.setResultMsg("读取Excel出现内部错误");
            return saveResult;
        }
        //目前资产导入只有一个Sheet页，所以只取第一个Sheet即可
        Sheet sheet = workbook.getSheetAt(0);
        if (sheet == null) {
            saveResult.setResult(false);
            saveResult.setResultMsg("读取Excel出现内部错误");
            return saveResult;
        }

        List<CitEAMAssetsDisposal> citEAMAssetsList = new ArrayList<>();
        List<CitDataImportLog> citDataImportLogList = new ArrayList<>();
//        Integer period = CitCommonUtil.getPeriod();

        //通过循环来完成sheet获取每一行每一列数据并进行相关逻辑处理
        for (int rowNum = sheet.getFirstRowNum() + 3; rowNum <= sheet.getLastRowNum(); rowNum++) {

            //获取该行数据
            Row rowData = sheet.getRow(rowNum);

            CitDataImportLog citDataImportLogError = generalCitDataImportLog("", "", "",
                    EnumCitImportType.EAMAssetsDisposal.getCode(), 0,
                    0, 0,
                    "", "EAM资产处置金额记录表",  false);
            citDataImportLogError.setRecordSize(1);

            CitEAMAssetsDisposal citEAMAssetsDisposal = new CitEAMAssetsDisposal();
            citEAMAssetsDisposal.setId(idService.nextId());
            citEAMAssetsDisposal.setPeriod(Integer.valueOf(CitCommonUtil.getPeriod().toString()));

            String[] split = CitCommonUtil.getValue(rowData.getCell(1)).toString().split("-");
            StringBuilder sb = new StringBuilder(split[0]);
            if(new Integer(split[1]) + 1 < 10){
                sb.append(0);
            }
            sb.append(split[1]);
            citEAMAssetsDisposal.setOccurPeriod(new Integer(sb.toString()));
            citEAMAssetsDisposal.setAssetLabelNumber(CitCommonUtil.getValue(rowData.getCell(6)).toString());
            citEAMAssetsDisposal.setCompensationSaleAmount(new BigDecimal(CitCommonUtil.getValue(rowData.getCell(7)).toString()));
            citEAMAssetsList.add(citEAMAssetsDisposal);
        }

        String msg = "追加导入成功";
        if (EnumTbImportType.CoverImport.getCode().equals(importType)) {
            msg = "覆盖导入成功";
            CitEAMAssetsDisposalExample example = new CitEAMAssetsDisposalExample();
            CitEAMAssetsDisposalExample.Criteria criteria = example.createCriteria();
            criteria.andCreatedByEqualTo(authUserHelper.getCurrentAuditor().get());
            criteria.andPeriodEqualTo(period);
            citEAMAssetsMapper.deleteByExample(example);
            updateImportLog(1);
        }

        int insertBatchNum = citEAMAssetsMapper.insertBatch(citEAMAssetsList);
        CitDataImportLog citDataImportLog = generalCitDataImportLog("", "", "",
                EnumCitImportType.EAMAssetsDisposal.getCode(), period / 100,
                0, period % 100,
                "", "EAM资产处置金额记录表", true);
        citDataImportLog.setRecordSize(insertBatchNum);
        citDataImportLog.setErrorMsg(msg);
        citDataImportLogList.add(citDataImportLog);
        addDataImportLog(citDataImportLogList);
        return null;
    }

    /**
     * 导入预提重分类数据
     * @param inputStream
     * @param fileName
     * @param period
     * @param importType
     * @return
     * @throws IOException
     * @throws InvalidFormatException
     * @throws ParseException
     */
    public OperationResultDto importSalaryAdvance(InputStream inputStream, List<String> orgList, String fileName, Integer period, Integer importType) throws IOException, InvalidFormatException, ParseException {
        OperationResultDto<Object> saveResult = new OperationResultDto<>();
        Workbook workbook = fileService.getWorkbook(inputStream, fileName, "citTrialBalance");
        if (workbook == null) {
            saveResult.setResult(false);
            saveResult.setResultMsg("读取Excel出现内部错误");
            return saveResult;
        }
        //目前资产导入只有一个Sheet页，所以只取第一个Sheet即可
        Sheet sheet = workbook.getSheetAt(0);
        if (sheet == null) {
            saveResult.setResult(false);
            saveResult.setResultMsg("读取Excel出现内部错误");
            return saveResult;
        }

        List<CitSalaryAdvance> citSalaryAdvanceList = new ArrayList<>();
        List<CitDataImportLog> citDataImportLogList = new ArrayList<>();
//        Integer period = CitCommonUtil.getPeriod();

        Map<String, Integer> companySuccessMap = new HashMap();
        Map<String, Integer> companyFailMap = new HashMap();
        Map<String, Integer> noCompanyMap = new HashMap();
        //通过循环来完成sheet获取每一行每一列数据并进行相关逻辑处理
        for (int rowNum = sheet.getFirstRowNum() + 2; rowNum <= sheet.getLastRowNum(); rowNum++) {

            //获取该行数据
            Row rowData = sheet.getRow(rowNum);

            CitDataImportLog citDataImportLogError = generalCitDataImportLog("", "", "",
                    EnumCitImportType.SalaryAdvance.getCode(), 0,
                    0, 0,
                    "", "预提重分类数据源", false);
            citDataImportLogError.setRecordSize(1);

            Object cellValueCode = CitCommonUtil.getValue(rowData.getCell(5));
            if(cellValueCode == null){
                citDataImportLogError.setErrorMsg("第"+(rowNum+1)+"行PO主体CODE为空");
                citDataImportLogList.add(citDataImportLogError);
                continue;
            }
            String companyCode = cellValueCode.toString();
            //根据code（主体）获取机构相关信息
            List<Organization> organizations = getOrganizationByCode(companyCode);
            String companyName = "";
            String orgId = "";
            String taxPayerId = "";
            //判断是否查询到该机构，若数据库中不存在则不执行导入并记录日志
            if (organizations != null && organizations.size() > 0) {

                orgId = organizations.get(0).getId();
                taxPayerId = StringUtils.isNotEmpty(organizations.get(0).getLegalCode()) ?
                        organizations.get(0).getLegalCode() : organizations.get(0).getTaxPayerNumber();

                companyName = organizations.get(0).getName();

                //拼接公司相关信息，作为键，其Excel出现的数量作为值
                StringBuilder sb = new StringBuilder(companyCode);
                sb.append(",");
                sb.append(orgId);
                sb.append(",");
                sb.append(taxPayerId);
                sb.append(",");
                sb.append(companyName);
                if(!orgList.contains(orgId)){
                    companyFailMap.put(sb.toString(),companyFailMap.get(sb.toString())==null?1:companyFailMap.get(sb.toString())+1);
                    continue;
                }
                companySuccessMap.put(sb.toString(),companySuccessMap.get(sb.toString())==null?1:companySuccessMap.get(sb.toString())+1);

            }else{
                noCompanyMap.put(companyCode,companySuccessMap.get(companyCode)==null?1:companySuccessMap.get(companyCode)+1);
                continue;
            }

            CitSalaryAdvance citSalaryAdvance = new CitSalaryAdvance();
            citSalaryAdvance.setId(idService.nextId());
            citSalaryAdvance.setPeriod(Integer.valueOf(CitCommonUtil.getPeriod().toString()));

            Object cellValue = CitCommonUtil.getValue(rowData.getCell(0));
            if(cellValue == null){
                citDataImportLogError.setErrorMsg("第"+(rowNum+1)+"行PO号为空");
                citDataImportLogList.add(citDataImportLogError);
                continue;
            }
            citSalaryAdvance.setPoNo(cellValue.toString());

            citSalaryAdvance.setPoSubjectCode(cellValueCode.toString());

            cellValue = CitCommonUtil.getValue(rowData.getCell(6));
            if(cellValue == null){
                citDataImportLogError.setErrorMsg("第"+(rowNum+1)+"行PO主体为空");
                citDataImportLogList.add(citDataImportLogError);
                continue;
            }
            citSalaryAdvance.setPoSubjectName(CitCommonUtil.getValue(rowData.getCell(6)).toString());

            cellValue = CitCommonUtil.getValue(rowData.getCell(16));
            if(cellValue == null){
                citDataImportLogError.setErrorMsg("第"+(rowNum+1)+"行预提为空");
                citDataImportLogList.add(citDataImportLogError);
                continue;
            }
            citSalaryAdvance.setAdvance(new BigDecimal(CitCommonUtil.getValue(rowData.getCell(16)).toString()));
            citSalaryAdvance.setApprovedStandardInvoiceAmount(new BigDecimal(CitCommonUtil.getValue(rowData.getCell(18)).toString()));

            citSalaryAdvanceList.add(citSalaryAdvance);
        }

        //判断导入的方式
        String msg = "追加导入成功";
        if (EnumTbImportType.CoverImport.getCode().equals(importType)) {
            //覆盖导入要删除原有的该操作人在当前期间导入的数据，且更改之前导入的日志记录状态
            msg = "覆盖导入成功";
            CitSalaryAdvanceExample example = new CitSalaryAdvanceExample();
            CitSalaryAdvanceExample.Criteria criteria = example.createCriteria();
            criteria.andCreatedByEqualTo(authUserHelper.getCurrentAuditor().get());
            criteria.andPeriodEqualTo(period);
            citSalaryAdvanceMapper.deleteByExample(example);
            updateImportLog(1);
        }
        int insertBatchNum = citSalaryAdvanceMapper.insertBatch(citSalaryAdvanceList);

        //循环遍历成功导入的数据，组装日志记录实体并放入集合
        for (Map.Entry<String, Integer> entry : companySuccessMap.entrySet()) {
            String[] split = entry.getKey().split(",");
            CitDataImportLog citDataImportLog = generalCitDataImportLog(split[0], split[1], split[2],
                    EnumCitImportType.SalaryAdvance.getCode(), period / 100,
                    0, period % 100,
                    split[3], "预提重分类数据源",  true);
            citDataImportLog.setRecordSize(entry.getValue());
            citDataImportLog.setErrorMsg(msg);
            citDataImportLogList.add(citDataImportLog);
        }
        for (Map.Entry<String, Integer> entry : companyFailMap.entrySet()) {
            String[] split = entry.getKey().split(",");
            CitDataImportLog citDataImportLogError = generalCitDataImportLog(split[0], split[1], split[2],
                    EnumCitImportType.SalaryAdvance.getCode(), period / 100,
                    0, period % 100,
                    split[3], "预提重分类数据源",  false);
            citDataImportLogError.setRecordSize(entry.getValue());
            citDataImportLogError.setErrorMsg(ErrorMessageCN.DoNotSelectCompany);
            citDataImportLogList.add(citDataImportLogError);
        }
        for (Map.Entry<String, Integer> entry : noCompanyMap.entrySet()) {
            CitDataImportLog citDataImportLogError = generalCitDataImportLog(entry.getKey(), "", "",
                    EnumCitImportType.SalaryAdvance.getCode(), 0,
                    0, 0,
                    "", "预提重分类数据源", false);
            citDataImportLogError.setRecordSize(entry.getValue());
            citDataImportLogError.setErrorMsg(ErrorMessageCN.NoCompanyError);
            citDataImportLogList.add(citDataImportLogError);
        }
        if (EnumTbImportType.CoverImport.getCode().equals(importType)) {
            updateImportLog(1);
        }
        addDataImportLog(citDataImportLogList);
        return null;
    }

    /**
     * 获取导入的日记账数据
     *
     * @return
     */
    public List<CitJournalEntryAdjust> getJournalAdjust() {
        CitJournalEntryAdjustExample example = new CitJournalEntryAdjustExample();
        CitJournalEntryAdjustExample.Criteria criteria = example.createCriteria();
        List<CitJournalEntryAdjust> citJournalEntryAdjusts = citJournalEntryAdjustMapper.selectByExample(example);
        return citJournalEntryAdjusts;
    }

    /**
     * 获取导入的试算平衡表数据
     *
     * @return
     */
    public List<CitJournalEntryAdjust> getTrialBalance() {
        CitJournalEntryAdjustExample example = new CitJournalEntryAdjustExample();
        CitJournalEntryAdjustExample.Criteria criteria = example.createCriteria();
        List<CitJournalEntryAdjust> citJournalEntryAdjusts = citJournalEntryAdjustMapper.selectByExample(example);
        return citJournalEntryAdjusts;
    }


    private BigDecimal add(BigDecimal v1, BigDecimal v2) {
        v1 = null == v1 ? BigDecimal.ZERO : v1;
        v2 = null == v2 ? BigDecimal.ZERO : v2;
        return v1.add(v2);
    }

    private CitDataImportLog generalCitDataImportLog(String companyCode, String orgId, String taxPayerId, Integer type, Integer periodYear,
                                                     Integer tmsPeriodMonth, Integer periodMonth, String companyName, String fileType, boolean importResult) {
        CitDataImportLog citDataImportLog = new CitDataImportLog();
        citDataImportLog.setId(idService.nextId());
        citDataImportLog.setTaxpayerIdNum(taxPayerId);
        citDataImportLog.setType(type);
        citDataImportLog.setOrganizationId(orgId);
        citDataImportLog.setCompanyCode(companyCode);
        citDataImportLog.setCompanyName(companyName);
        citDataImportLog.setFileType(fileType);
        citDataImportLog.setPeriodYear(periodYear);
        citDataImportLog.setTmsPeriodMonth(tmsPeriodMonth);
        citDataImportLog.setPeriodMonth(periodMonth);
        citDataImportLog.setPeriodStatus("人工导入");
        citDataImportLog.setOperator(authUserHelper.getCurrentAuditor().get());
        citDataImportLog.setImportResult(importResult);
        return citDataImportLog;
    }

    /**
     * 根据文件类型修改版本状态，置为历史版本
     * @param type
     * @return
     */
    private int updateImportLog(Integer type) {
        int updateDisplayOld = citDataImportLogMapper.updateDisplayOld(type);
        return updateDisplayOld;
    }

    private int addDataImportLog(List<CitDataImportLog> dataImportLogs) {
        //在添加import log前先将原来导入的数据版本置为旧版本
//        citDataImportLogMapper.updateDisplayOld(dataImportLogs.get(0).getType());
        int res = 0;
        for (CitDataImportLog dataImportLog : dataImportLogs) {
            res += citDataImportLogMapper.insertSelective(dataImportLog);
        }
        return res;
    }

    /**
     * 根据导入类型获取导入记录
     *
     * @param type
     * @return
     */
    public List<CitDataImportLogDto> getCitDataImportLog(Integer type) {
        List<CitDataImportLogDto> citDataImportLogDtos = Lists.newArrayList();
        CitDataImportLogExample citDataImportLogExample = new CitDataImportLogExample();
        //判断传入的参数是否可用
        if (EnumImportType.Undefined.getCode().equals(type)) {
            citDataImportLogExample.setOrderByClause("update_time desc");
        } else {
            citDataImportLogExample.createCriteria().andTypeEqualTo(type).andDisplayEqualTo(true);
            citDataImportLogExample.setOrderByClause("update_time desc");
        }
        //把从数据库获取的实体转化成页面Dto
        citDataImportLogMapper.selectByExample(citDataImportLogExample).forEach(citDataImportLog -> {
            CitDataImportLogDto dataImportLogDto = new CitDataImportLogDto();
            beanUtil.copyProperties(citDataImportLog, dataImportLogDto);
            citDataImportLogDtos.add(dataImportLogDto);
        });
        return citDataImportLogDtos;
    }

    private boolean isSheetEmpty(Sheet sheet) {
        if (null == sheet.getRow(0).getCell(0)) {
            return true;
        }
        return false;
    }

    /**
     * 根据公司code获取机构相关信息
     * @param companyCode
     * @return
     */
    private List<Organization> getOrganizationByCode(String companyCode) {
        OrganizationExample example = new OrganizationExample();
        example.createCriteria().andCodeEqualTo(companyCode);
        List<Organization> organizations = organizationMapper.selectByExample(example);
        return organizations;
    }
    /**
     * 根据公司code获取机构相关信息
     * @param companyName
     * @return
     */
    private List<Organization> getOrganizationByName(String companyName) {
        OrganizationExample example = new OrganizationExample();
        example.createCriteria().andNameEqualTo(companyName);
        List<Organization> organizations = organizationMapper.selectByExample(example);
        return organizations;
    }

}
